angular.module('PasswordMonitor').factory('HashingService', function ($q) {

    var size = 6;
    var workerQueue = [];
    var taskQueue = [];

    var Salt = new (function () {
        var salt;

        function Salt() {
            chrome.storage.local.get('salt', function (result) {
                if (angular.isDefined(result['salt'])) {
                    salt = result['salt'];
                }
            });
        }

        Salt.prototype.getSalt = function () {
            // Let's check if we've retrieved salt already
            if (angular.isDefined(salt)) {
                return $q.when(salt);
            }

            var deferred = $q.defer();
            // If we haven't, let's see if it's stored
            chrome.storage.local.get('salt', function (result) {
                if (angular.isDefined(result['salt'])) {
                    salt = result['salt'];
                    deferred.resolve(salt);
                }
                else {
                    deferred.resolve(false);
                }
            });
            return deferred.promise;
        };
        return Salt;
    }());

    // Create all the workers without starting them
    for (var n = 0; n < size; n++) {
        workerQueue.push(new WebWorker());
    }

    // When a worker is free we either pick up a new task, or queue for later use
    function freeWorker(worker) {
        if (taskQueue.length > 0) {
            var newTask = taskQueue.shift();
            worker.run(newTask);
        }
        else {
            workerQueue.push(worker);
        }
    }

    function WebWorker() {
        var this_ = this;
        this.worker = new Worker("../js/hashWorker.js");
        this.run = function (task) {
            this_.worker.onmessage = function (event) {
                var promise = task['promise'];
                promise.resolve(event.data);
                freeWorker(this_);
            };
            this_.worker.postMessage({
                cleartext: task['cleartext'],
                salt: task['salt'] === undefined ? Salt.getSalt() : task['salt']
            });
        }
    }

    return {
        hashThis: function (cleartext, providedSalt) {
            var deferred = $q.defer();
            var task = {cleartext: cleartext, salt: providedSalt, promise: deferred};
            if (workerQueue.length > 0) {
                var worker = workerQueue.shift();
                worker.run(task);
            }
            else {
                taskQueue.push(task);
            }
            return deferred.promise;
        },

        getSalt: Salt.getSalt
    }
});